@@ -512,7 +512,7 @@ describe("cpp_register", {
 "# Generated by cpp11: do not edit by hand
 
 foo <- function() {
-  .Call(\"_testPkg_foo\")
+  .Call(\"_testPkg_foo\", PACKAGE = \"testPkg\")
 }
 ")
     cpp_bindings <- file.path(p, "src", "cpp11.cpp")
@@ -585,6 +585,25 @@ extern \"C\" void R_init_testPkg(DllInfo* dll){
     )
   })
 
+  it("includes pkg_types.hpp if included in src", {
+    pkg <- local_package()
+    p <- pkg_path(pkg)
+    dir.create(file.path(p, "src"))
+    file.copy(test_path("single.cpp"), file.path(p, "src", "single.cpp"))
+    writeLines("#include <sstream>", file.path(p, "src", "testPkg_types.hpp"))
+    cpp_register(p)
+
+    expect_true(
+      any(
+        grepl(
+          pattern = '#include "testPkg_types.hpp"',
+          x = readLines(file.path(p, "src", "cpp11.cpp")),
+          fixed = TRUE
+        )
+      )
+    )
+  })
+
   it("includes pkg_types.h if included in inst/include", {
     pkg <- local_package()
     p <- pkg_path(pkg)
@@ -606,6 +625,27 @@ extern \"C\" void R_init_testPkg(DllInfo* dll){
     )
   })
 
+  it("includes pkg_types.hpp if included in inst/include", {
+    pkg <- local_package()
+    p <- pkg_path(pkg)
+    dir.create(file.path(p, "src"))
+    file.copy(test_path("single.cpp"), file.path(p, "src", "single.cpp"))
+
+    dir.create(file.path(p, "inst", "include"), recursive = TRUE)
+    writeLines("#include <sstream>", file.path(p, "inst", "include", "testPkg_types.hpp"))
+    cpp_register(p)
+
+    expect_true(
+      any(
+        grepl(
+          pattern = '#include "testPkg_types.hpp"',
+          x = readLines(file.path(p, "src", "cpp11.cpp")),
+          fixed = TRUE
+        )
+      )
+    )
+  })
+
   it("does not error if no files have registered functions", {
     pkg <- local_package()
     p <- pkg_path(pkg)
